/* eslint-disable @typescript-eslint/no-explicit-any */
import { AxiosError } from 'axios';
import SlackHook from 'winston-slack-webhook-transport';

import { LOG_LEVEL } from '#/logger/logger.types';

import { AttachementField, Attachment, ErrorMessageWithAttachment } from './slack.transport.types';

type InfoData = Error | AxiosError | unknown | unknown[];

export const getMessageForSlack = (info: any, nodeEnv: string): ErrorMessageWithAttachment | object => {
	const errorObject = {
		context: info.context ?? undefined,
		message: info.message ?? undefined,
		level: (info.level as string) ?? undefined,
		splat: info[Symbol.for('splat')] ?? undefined,
		data: info.data ?? undefined,
		timestamp: (info.timestamp as string) ?? undefined,
		symbolMessage: JSON.parse(info[Symbol.for('message')]) ?? undefined,
	} as {
		context: string | undefined;
		message: string | undefined;
		level: keyof typeof LOG_LEVEL;
		splat: symbol | undefined;
		data: InfoData | undefined;
		timestamp: string | undefined;
		symbolMessage: unknown | undefined;
	};

	const { data, context, message } = errorObject;
	let errorMessage = message;

	const attachments: Attachment[] = [];
	const fields: AttachementField[] = [];

	if (data !== undefined) {
		if (data instanceof Error || data instanceof AxiosError) {
			errorMessage = data.message;
			if (data.stack !== undefined) {
				const stackParts = data.stack.split('\n');

				stackParts.forEach((part) =>
					fields.push({
						title: '',
						value: part.trim(),
						short: false,
					})
				);
			}
		}
	}

	attachments.push({
		// green
		color: '#36a64f',
		fields,
		footer: `*${errorObject.timestamp} - ${nodeEnv}*`.toUpperCase(),
	});

	const slackMessage = {
		attachments,
		blocks: [
			/**
			 * Header
			 */
			{
				type: 'header',
				text: {
					type: 'plain_text',
					text: ':x:The service got an error :x:',
					emoji: true,
				},
			},
			/**
			 * Level
			 */
			{
				type: 'context',
				elements: [
					{
						type: 'mrkdwn',
						text: `:exclamation:Error level: *${errorObject.level}*`.toUpperCase() + `:exclamation:`,
					},
				],
			},
			{
				type: 'divider',
			},
			/**
			 * Context  Message
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Context Message:*',
					},
					{
						type: 'mrkdwn',
						text: message,
					},
				],
			},
			{
				type: 'divider',
			},
			/**
			 * Context
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Context:*',
					},
					{
						type: 'mrkdwn',
						text: context,
					},
				],
			},
			{
				type: 'divider',
			},

			/**
			 * Error Stack or Response
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Message:*',
					},
					{
						type: 'mrkdwn',
						text: errorMessage,
					},
				],
			},
		],
	};
	return slackMessage;
};

export default class SlackTransport {
	public static create(nodeEnv: string, webhookUrl: string) {
		return new SlackHook({
			level: LOG_LEVEL.Error || LOG_LEVEL.Fatal || LOG_LEVEL.Emergency,
			webhookUrl,
			formatter: (info: any) => getMessageForSlack(info, nodeEnv),
		});
	}
}
